@@ -21,21 +21,23 @@
 
 #include "kbookmarkmanager.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtCore/QProcess>
-#include <QtCore/QRegExp>
-#include <QtCore/QTextStream>
+#include "kbookmarks_debug.h"
+#include <QDir>
+#include <QFile>
+#include <QFileInfo>
+#include <QProcess>
+#include <QRegularExpression>
+#include <QTextStream>
 #include <QTextCodec>
+#ifndef KBOOKMARKS_NO_DBUS
 #include <QDBusConnection>
+#include <QDBusMessage>
+#endif
 #include <QMessageBox>
 #include <QApplication>
 #include <QReadWriteLock>
 #include <QThread>
 
-#include <QDBusMessage>
 #include <kbackup.h>
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -47,17 +49,29 @@
 #include "kbookmarkmenu_p.h"
 #include "kbookmarkimporter.h"
 #include "kbookmarkdialog.h"
+#ifndef KBOOKMARKS_NO_DBUS
 #include "kbookmarkmanageradaptor_p.h"
+#endif
 
-#define BOOKMARK_CHANGE_NOTIFY_INTERFACE "org.kde.KIO.KBookmarkManager"
+namespace {
+namespace Strings {
+QString bookmarkChangeNotifyInterface() { return QStringLiteral("org.kde.KIO.KBookmarkManager"); }
+QString piData() { return QStringLiteral("version=\"1.0\" encoding=\"UTF-8\""); }
+}
+}
 
 class KBookmarkManagerList : public QList<KBookmarkManager *>
 {
 public:
+    KBookmarkManagerList();
     ~KBookmarkManagerList()
     {
+        cleanup();
+    }
+    void cleanup() {
         QList<KBookmarkManager *> copy = *this;
         qDeleteAll(copy); // auto-delete functionality
+        clear();
     }
 
     QReadWriteLock lock;
@@ -65,6 +79,18 @@ public:
 
 Q_GLOBAL_STATIC(KBookmarkManagerList, s_pSelf)
 
+static void deleteManagers() {
+    if (s_pSelf.exists()) {
+        s_pSelf->cleanup();
+    }
+}
+
+KBookmarkManagerList::KBookmarkManagerList()
+{
+    // Delete the KBookmarkManagers while qApp exists, since we interact with the DBus thread
+    qAddPostRoutine(deleteManagers);
+}
+
 class KBookmarkMap : private KBookmarkGroupTraverser
 {
 public:
@@ -79,12 +105,12 @@ public:
         return m_bk_map.value(url);
     }
 private:
-    void visit(const KBookmark &) Q_DECL_OVERRIDE;
-    void visitEnter(const KBookmarkGroup &) Q_DECL_OVERRIDE
+    void visit(const KBookmark &) override;
+    void visitEnter(const KBookmarkGroup &) override
     {
         ;
     }
-    void visitLeave(const KBookmarkGroup &) Q_DECL_OVERRIDE
+    void visitLeave(const KBookmarkGroup &) override
     {
         ;
     }
@@ -124,10 +150,10 @@ public:
         , m_docIsLoaded(bDocIsloaded)
         , m_update(false)
         , m_dialogAllowed(true)
-        , m_dialogParent(0)
+        , m_dialogParent(nullptr)
         , m_browserEditor(false)
         , m_typeExternal(false)
-        , m_dirWatch(0)
+        , m_dirWatch(nullptr)
     {}
 
     ~KBookmarkManagerPrivate()
@@ -164,12 +190,12 @@ static KBookmarkManager *lookupExisting(const QString &bookmarksFile)
             return *bmit;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 KBookmarkManager *KBookmarkManager::managerForFile(const QString &bookmarksFile, const QString &dbusObjectName)
 {
-    KBookmarkManager *mgr(0);
+    KBookmarkManager *mgr(nullptr);
     {
         QReadLocker readLock(&s_pSelf()->lock);
         mgr = lookupExisting(bookmarksFile);
@@ -191,7 +217,7 @@ KBookmarkManager *KBookmarkManager::managerForFile(const QString &bookmarksFile,
 
 KBookmarkManager *KBookmarkManager::managerForExternalFile(const QString &bookmarksFile)
 {
-    KBookmarkManager *mgr(0);
+    KBookmarkManager *mgr(nullptr);
     {
         QReadLocker readLock(&s_pSelf()->lock);
         mgr = lookupExisting(bookmarksFile);
@@ -219,8 +245,6 @@ KBookmarkManager *KBookmarkManager::createTempManager()
     return mgr;
 }
 
-#define PI_DATA "version=\"1.0\" encoding=\"UTF-8\""
-
 static QDomElement createXbelTopLevelElement(QDomDocument &doc)
 {
     QDomElement topLevel = doc.createElement(QStringLiteral("xbel"));
@@ -228,7 +252,7 @@ static QDomElement createXbelTopLevelElement(QDomDocument &doc)
     topLevel.setAttribute(QStringLiteral("xmlns:bookmark"), QStringLiteral("http://www.freedesktop.org/standards/desktop-bookmarks"));
     topLevel.setAttribute(QStringLiteral("xmlns:kdepriv"), QStringLiteral("http://www.kde.org/kdepriv"));
     doc.appendChild(topLevel);
-    doc.insertBefore(doc.createProcessingInstruction(QStringLiteral("xml"), PI_DATA), topLevel);
+    doc.insertBefore(doc.createProcessingInstruction(QStringLiteral("xml"), Strings::piData()), topLevel);
     return topLevel;
 }
 
@@ -240,7 +264,7 @@ KBookmarkManager::KBookmarkManager(const QString &bookmarksFile, const QString &
             parse();    //sets d->m_dbusObjectName
         }
 
-    init("/KBookmarkManager/" + d->m_dbusObjectName);
+    init(QLatin1String("/KBookmarkManager/") + d->m_dbusObjectName);
 
     d->m_update = true;
 
@@ -281,7 +305,7 @@ KBookmarkManager::KBookmarkManager(const QString &bookmarksFile)
     QObject::connect(d->m_dirWatch, &KDirWatch::deleted,
             this, &KBookmarkManager::slotFileChanged);
 
-    // qDebug() << "starting KDirWatch for" << d->m_bookmarksFile;
+    // qCDebug(KBOOKMARKS_LOG) << "starting KDirWatch for" << d->m_bookmarksFile;
 }
 
 KBookmarkManager::KBookmarkManager()
@@ -295,23 +319,42 @@ KBookmarkManager::KBookmarkManager()
 
 void KBookmarkManager::init(const QString &dbusPath)
 {
+#ifndef KBOOKMARKS_NO_DBUS
     // A KBookmarkManager without a dbus name is a temporary one, like those used by importers;
     // no need to register them to dbus
     if (dbusPath != QLatin1String("/KBookmarkManager/") && dbusPath != QLatin1String("/KBookmarkManager/generated")) {
         new KBookmarkManagerAdaptor(this);
         QDBusConnection::sessionBus().registerObject(dbusPath, this);
 
-        QDBusConnection::sessionBus().connect(QString(), dbusPath, BOOKMARK_CHANGE_NOTIFY_INTERFACE,
+        QDBusConnection::sessionBus().connect(QString(), dbusPath, Strings::bookmarkChangeNotifyInterface(),
                                               QStringLiteral("bookmarksChanged"), this, SLOT(notifyChanged(QString,QDBusMessage)));
-        QDBusConnection::sessionBus().connect(QString(), dbusPath, BOOKMARK_CHANGE_NOTIFY_INTERFACE,
+        QDBusConnection::sessionBus().connect(QString(), dbusPath, Strings::bookmarkChangeNotifyInterface(),
                                               QStringLiteral("bookmarkConfigChanged"), this, SLOT(notifyConfigChanged()));
     }
+#endif
+}
+
+void KBookmarkManager::startKEditBookmarks(const QStringList &args)
+{
+    bool success = QProcess::startDetached(QStringLiteral(KEDITBOOKMARKS_BINARY), args);
+
+    if (!success) {
+        QString err = tr("Cannot launch keditbookmarks.\n\n"
+                         "Most likely you do not have keditbookmarks currently installed");
+
+        if (d->m_dialogAllowed && qobject_cast<QApplication *>(qApp) && QThread::currentThread() == qApp->thread()) {
+            QMessageBox::warning(QApplication::activeWindow(), QApplication::applicationName(), err);
+        }
+
+        qCWarning(KBOOKMARKS_LOG) << QStringLiteral("Failed to start keditbookmarks");
+        emit this->error(err);
+    }
 }
 
 void KBookmarkManager::slotFileChanged(const QString &path)
 {
     if (path == d->m_bookmarksFile) {
-        // qDebug() << "file changed (KDirWatch) " << path ;
+        // qCDebug(KBOOKMARKS_LOG) << "file changed (KDirWatch) " << path ;
         // Reparse
         parse();
         // Tell our GUI
@@ -357,17 +400,17 @@ QDomDocument KBookmarkManager::internalDocument() const
 void KBookmarkManager::parse() const
 {
     d->m_docIsLoaded = true;
-    // qDebug() << "KBookmarkManager::parse " << d->m_bookmarksFile;
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::parse " << d->m_bookmarksFile;
     QFile file(d->m_bookmarksFile);
     if (!file.open(QIODevice::ReadOnly)) {
-        qWarning() << "Can't open " << d->m_bookmarksFile;
+        qCWarning(KBOOKMARKS_LOG) << "Can't open " << d->m_bookmarksFile;
         return;
     }
     d->m_doc = QDomDocument(QStringLiteral("xbel"));
     d->m_doc.setContent(&file);
 
     if (d->m_doc.documentElement().isNull()) {
-        qWarning() << "KBookmarkManager::parse : main tag is missing, creating default " << d->m_bookmarksFile;
+        qCWarning(KBOOKMARKS_LOG) << "KBookmarkManager::parse : main tag is missing, creating default " << d->m_bookmarksFile;
         QDomElement element = d->m_doc.createElement(QStringLiteral("xbel"));
         d->m_doc.appendChild(element);
     }
@@ -376,7 +419,7 @@ void KBookmarkManager::parse() const
 
     QString mainTag = docElem.tagName();
     if (mainTag != QLatin1String("xbel")) {
-        qWarning() << "KBookmarkManager::parse : unknown main tag " << mainTag;
+        qCWarning(KBOOKMARKS_LOG) << "KBookmarkManager::parse : unknown main tag " << mainTag;
     }
 
     if (d->m_dbusObjectName.isNull()) {
@@ -393,7 +436,7 @@ void KBookmarkManager::parse() const
     }
 
     QDomProcessingInstruction pi;
-    pi = d->m_doc.createProcessingInstruction(QStringLiteral("xml"), PI_DATA);
+    pi = d->m_doc.createProcessingInstruction(QStringLiteral("xml"), Strings::piData());
     d->m_doc.insertBefore(pi, docElem);
 
     file.close();
@@ -408,7 +451,7 @@ bool KBookmarkManager::save(bool toolbarCache) const
 
 bool KBookmarkManager::saveAs(const QString &filename, bool toolbarCache) const
 {
-    // qDebug() << "KBookmarkManager::save " << filename;
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::save " << filename;
 
     // Save the bookmark toolbar folder for quick loading
     // but only when it will actually make things quicker
@@ -448,13 +491,13 @@ bool KBookmarkManager::saveAs(const QString &filename, bool toolbarCache) const
         QString err = tr("Unable to save bookmarks in %1. Reported error was: %2. "
                          "This error message will only be shown once. The cause "
                          "of the error needs to be fixed as quickly as possible, "
-                         "which is most likely a full hard drive.").arg(filename).arg(file.errorString());
+                         "which is most likely a full hard drive.").arg(filename, file.errorString());
 
         if (d->m_dialogAllowed && qobject_cast<QApplication *>(qApp) && QThread::currentThread() == qApp->thread()) {
             QMessageBox::critical(QApplication::activeWindow(), QApplication::applicationName(), err);
         }
 
-        qCritical() << QStringLiteral("Unable to save bookmarks in %1. File reported the following error-code: %2.").arg(filename).arg(file.error());
+        qCCritical(KBOOKMARKS_LOG) << QStringLiteral("Unable to save bookmarks in %1. File reported the following error-code: %2.").arg(filename).arg(file.error());
         emit const_cast<KBookmarkManager *>(this)->error(err);
     }
     hadSaveError = true;
@@ -473,27 +516,27 @@ KBookmarkGroup KBookmarkManager::root() const
 
 KBookmarkGroup KBookmarkManager::toolbar()
 {
-    // qDebug() << "KBookmarkManager::toolbar begin";
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::toolbar begin";
     // Only try to read from a toolbar cache if the full document isn't loaded
     if (!d->m_docIsLoaded) {
-        // qDebug() << "KBookmarkManager::toolbar trying cache";
+        // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::toolbar trying cache";
         const QString cacheFilename = d->m_bookmarksFile + QLatin1String(".tbcache");
         QFileInfo bmInfo(d->m_bookmarksFile);
         QFileInfo cacheInfo(cacheFilename);
         if (d->m_toolbarDoc.isNull() &&
                 QFile::exists(cacheFilename) &&
                 bmInfo.lastModified() < cacheInfo.lastModified()) {
-            // qDebug() << "KBookmarkManager::toolbar reading file";
+            // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::toolbar reading file";
             QFile file(cacheFilename);
 
             if (file.open(QIODevice::ReadOnly)) {
                 d->m_toolbarDoc = QDomDocument(QStringLiteral("cache"));
                 d->m_toolbarDoc.setContent(&file);
-                // qDebug() << "KBookmarkManager::toolbar opened";
+                // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::toolbar opened";
             }
         }
         if (!d->m_toolbarDoc.isNull()) {
-            // qDebug() << "KBookmarkManager::toolbar returning element";
+            // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::toolbar returning element";
             QDomElement elem = d->m_toolbarDoc.firstChild().toElement();
             return KBookmarkGroup(elem);
         }
@@ -514,11 +557,15 @@ KBookmarkGroup KBookmarkManager::toolbar()
 
 KBookmark KBookmarkManager::findByAddress(const QString &address)
 {
-    // qDebug() << "KBookmarkManager::findByAddress " << address;
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::findByAddress " << address;
     KBookmark result = root();
     // The address is something like /5/10/2+
-    const QStringList addresses = address.split(QRegExp(QStringLiteral("[/+]")), QString::SkipEmptyParts);
-    // qWarning() << addresses.join(",");
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+    const QStringList addresses = address.split(QRegularExpression(QStringLiteral("[/+]")), QString::SkipEmptyParts);
+#else
+    const QStringList addresses = address.split(QRegularExpression(QStringLiteral("[/+]")), Qt::SkipEmptyParts);
+#endif
+    // qCWarning(KBOOKMARKS_LOG) << addresses.join(",");
     for (QStringList::const_iterator it = addresses.begin(); it != addresses.end();) {
         bool append = ((*it) == QLatin1String("+"));
         uint number = (*it).toUInt();
@@ -528,16 +575,16 @@ KBookmark KBookmarkManager::findByAddress(const QString &address)
         for (uint i = 0; ((i < number) || append) && !bk.isNull(); ++i) {
             lbk = bk;
             bk = group.next(bk);
-            // qWarning() << i;
+            // qCWarning(KBOOKMARKS_LOG) << i;
         }
         it++;
-        // qWarning() << "found section";
+        // qCWarning(KBOOKMARKS_LOG) << "found section";
         result = bk;
     }
     if (result.isNull()) {
-        qWarning() << "KBookmarkManager::findByAddress: couldn't find item " << address;
+        qCWarning(KBOOKMARKS_LOG) << "KBookmarkManager::findByAddress: couldn't find item " << address;
     }
-    // qWarning() << "found " << result.address();
+    // qCWarning(KBOOKMARKS_LOG) << "found " << result.address();
     return result;
 }
 
@@ -551,7 +598,7 @@ void KBookmarkManager::emitChanged(const KBookmarkGroup &group)
     (void) save(); // KDE5 TODO: emitChanged should return a bool? Maybe rename it to saveAndEmitChanged?
 
     // Tell the other processes too
-    // qDebug() << "KBookmarkManager::emitChanged : broadcasting change " << group.address();
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::emitChanged : broadcasting change " << group.address();
 
     emit bookmarksChanged(group.address());
 
@@ -570,7 +617,7 @@ void KBookmarkManager::notifyCompleteChange(const QString &caller)   // DBUS cal
         return;
     }
 
-    // qDebug() << "KBookmarkManager::notifyCompleteChange";
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::notifyCompleteChange";
     // The bk editor tells us we should reload everything
     // Reparse
     parse();
@@ -581,15 +628,16 @@ void KBookmarkManager::notifyCompleteChange(const QString &caller)   // DBUS cal
 
 void KBookmarkManager::notifyConfigChanged() // DBUS call
 {
-    // qDebug() << "reloaded bookmark config!";
+    // qCDebug(KBOOKMARKS_LOG) << "reloaded bookmark config!";
     KBookmarkSettings::self()->readSettings();
     parse(); // reload, and thusly recreate the menus
     emit configChanged();
 }
 
+#ifndef KBOOKMARKS_NO_DBUS
 void KBookmarkManager::notifyChanged(const QString &groupAddress, const QDBusMessage &msg)   // DBUS call
 {
-    // qDebug() << "KBookmarkManager::notifyChanged ( "<<groupAddress<<")";
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::notifyChanged ( "<<groupAddress<<")";
     if (!d->m_update) {
         return;
     }
@@ -600,11 +648,12 @@ void KBookmarkManager::notifyChanged(const QString &groupAddress, const QDBusMes
         parse();
     }
 
-    // qDebug() << "KBookmarkManager::notifyChanged " << groupAddress;
+    // qCDebug(KBOOKMARKS_LOG) << "KBookmarkManager::notifyChanged " << groupAddress;
     //KBookmarkGroup group = findByAddress( groupAddress ).toGroup();
     //Q_ASSERT(!group.isNull());
     emit changed(groupAddress, QString());
 }
+#endif
 
 void KBookmarkManager::setEditorOptions(const QString &caption, bool browser)
 {
@@ -625,7 +674,7 @@ void KBookmarkManager::slotEditBookmarks()
         args << QStringLiteral("--dbusObjectName") << d->m_dbusObjectName;
     }
     args << d->m_bookmarksFile;
-    QProcess::startDetached(QStringLiteral("keditbookmarks"), args);
+    startKEditBookmarks(args);
 }
 
 void KBookmarkManager::slotEditBookmarksAtAddress(const QString &address)
@@ -642,7 +691,7 @@ void KBookmarkManager::slotEditBookmarksAtAddress(const QString &address)
     }
     args << QStringLiteral("--address") << address
          << d->m_bookmarksFile;
-    QProcess::startDetached(QStringLiteral("keditbookmarks"), args);
+    startKEditBookmarks(args);
 }
 
 ///////
@@ -650,7 +699,7 @@ bool KBookmarkManager::updateAccessMetadata(const QString &url)
 {
     d->m_map.update(this);
     QList<KBookmark> list = d->m_map.find(url);
-    if (list.count() == 0) {
+    if (list.isEmpty()) {
         return false;
     }
 
@@ -686,7 +735,7 @@ KBookmarkManager *KBookmarkManager::userBookmarksManager()
     return bookmarkManager;
 }
 
-KBookmarkSettings *KBookmarkSettings::s_self = 0;
+KBookmarkSettings *KBookmarkSettings::s_self = nullptr;
 
 void KBookmarkSettings::readSettings()
 {
